#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import os

here = os.path.dirname(os.path.realpath(__file__))
versionfile = os.path.join(here, 'version')


try:
    import git
    #from git.exc import InvalidGitRepositoryError
    repo = git.Repo(search_parent_directories=False)
    __version__ = '-'.join(repo.git.describe('--tags').split('-')[:2])
    with open(versionfile, 'wt') as v:
        v.write(__version__)
#except ImportError as e:
except Exception as e:
    try:
        with open(versionfile, 'rt') as v:
            __version__ = v.read()
    except OSError as e:
        __version__ = ''
#except InvalidGitRepositoryError as e:






#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import unittest
from dnsuptools import dnsuptools
from tests.passwords import inwxUserDict, inwxPasswdDict

turl = "test23.bahn.cf"

class TestDNSUptoolsMiscFncs(unittest.TestCase):
    def testParseNSentry(self):
        x = {'name': 'testname', 'content': 'testcontent', 'ttl': 3600}
        y = dnsuptools.parseNSentry(x)
        self.assertEqual(y, {'ns': 'testcontent'})


class TestDNSUptools(unittest.TestCase):
    def setUp(self):
        self.dnsu = dnsuptools.DNSUpTools()



#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import time
from INWX.Domrobot import ApiClient

from simpleloggerplus import simpleloggerplus as log
from dnsuptools.dnshelpers import createKeyDomainIfNotExists

inwxUserDict = {'default': 'user'}
inwxPasswdDict = {'default': 'passwd'}

class INWXwrapper:
    '''Class allows updating inwx zone entries'''
    def __init__(self):
        global inwxUserDict
        global inwxPasswdDict
        self.__apiUrl = ApiClient.API_LIVE_URL
        self.__conn = None
        self.__userDict = {'default': 'user'}
        self.__passwdDict = {'default': 'passwd'}
        self.__rv = None
        self.__openedDomain = ''
        self.__isConnected = False
        self.__loggedInCredentials = {}
        self.setUserDict(inwxUserDict)
        self.setPasswdDict(inwxPasswdDict)

    def setApiUrl(self, apiUrl):
        self.disconnect()
        self.__apiUrl = apiUrl

    def setUser(self, user, domain = 'default'):
        self.__userDict[str(domain)] = user

    def setUserDict(self, userDict):
        self.__userDict = userDict

    def setPasswd(self, passwd, domain = 'default'):
        self.__passwdDict[str(domain)] = passwd

    def setPasswdDict(self, passwdDict):
        self.__passwdDict = passwdDict

    def getPasswd(self, domain):
        domain = str(domain)
        return getDictLike(self.__passwdDict, domain, self.__passwdDict['default'])

    def getUser(self, domain):
        domain = str(domain)
        return getDictLike(self.__userDict, domain, self.__userDict['default'])

    def connect(self):
        if self.__isConnected is True:
            return
        self.__conn = ApiClient(api_url=self.__apiUrl, debug_mode=False)
        self.__isConnected = True

    def disconnect(self):
        if self.__isConnected is False:
            return
        self.logout()
        self.__conn = None
        self.__isConnected = False

    def login(self, domain):
        if 'user' in self.__loggedInCredentials and 'pass' in self.__loggedInCredentials:
            if self.getUser(domain) == self.__loggedInCredentials['user'] and self.getPasswd(domain) == self.__loggedInCredentials['pass']:
                return
        self.connect()
        loggedInCredentials = {'lang': 'en', 'user': self.getUser(domain), 'pass': self.getPasswd(domain)}
        log.debug(loggedInCredentials)
        self.__rv = self.__conn.login(loggedInCredentials['user'], loggedInCredentials['pass'])
        log.debug(self.__rv)
        if 1000 != self.__rv['code']:
            return
        self.__loggedInCredentials = dict(loggedInCredentials)
        self.__openedDomain = str(domain)
        self.__isLoggedIn = True

    def logout(self):
        if self.__isLoggedIn is False:
            return
        self.__loggedInCredentials = {}
        self.__openedDomain = ''


    def autologin(self, recordDict):
        recordDict = dict(recordDict)
        createKeyDomainIfNotExists(recordDict)
        if 'domain' in recordDict:
            self.login(recordDict['domain'])



    # Yes, login also for info needed!
    def info(self, infoDict):
        self.autologin(infoDict)
        stateDict = {}
        caaWorkaroundPre(infoDict, stateDict)
        rv = self.callAPI('nameserver.info', infoDict)
        rv = rv['resData']
        if 'record' not in rv:
            return []
        rv = rv['record']
        caaWorkaroundPost(rv, stateDict)
        return rv

    def create(self, createDict):
        createKeyDomainIfNotExists(createDict)
        self.login(createDict['domain'])
        return self.callAPI('nameserver.createRecord', createDict)

    # warning: no autologin, if no domain and no name provided
    #          - that is when you support only the record id
    #          but should not be a problem, because you can only
    #          know record id after info() needing login, automatically
    #          happen by providing domain or name
    def delete(self, deleteDict):
        self.autologin(deleteDict)
        return self.callAPI('nameserver.deleteRecord', deleteDict)

    def update(self, updateDict):
        self.autologin(updateDict)
        if 'domain' in updateDict:
            del updateDict['domain']
        return self.callAPI('nameserver.updateRecord', updateDict)

    def callAPI(self, method, params):
        for i in range(10):
            if i > 0:
                log.info('INWX api wait {} s before retry'.format(i**3))
                time.sleep(i**3)
            rv = self.__conn.call_api(api_method=method, method_params=params)
            if 2400 == rv['code']:
                log.info('INWX api error code: {}'.format(rv['code']))
                continue
            if 2500 == rv['code']:
                log.info('INWX api error code: {}'.format(rv['code']))
                continue
            if 2502 == rv['code']:
                log.info('INWX api error code: {}'.format(rv['code']))
                continue
            return rv

        log.error('INWX api retry limit exceeded!')
        log.error('INWX api error code was: {}'.format(rv['code']))
        return rv

def caaWorkaroundPre(infoDict, stateDict):
    stateDict['iscaa'] = False
    if 'type' not in infoDict:
        return
    if 'CAA' != infoDict['type']:
        return
    stateDict['iscaa'] = True
    del infoDict['type']


def caaWorkaroundPost(rv, stateDict):
    if not stateDict['iscaa']:
        return
    rv[:] = [e for e in rv if 'CAA' == e['type']]

def getDictLike(dataDict, searchFor, default=None, splitChar='.'):
    splitList = searchFor.split(splitChar)
    for i in range(len(splitList)):
        subPart = splitChar.join(splitList[i:])
        if subPart in dataDict:
            return dataDict[subPart]
    return default




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

from OpenSSL import crypto
import hashlib

def tlsaFromCertFile(certFilename, certConstr = 3, keyOnly = 0, hashType = 1):
    with open(certFilename) as f:
        certCont = f.read()
    if 2 == int(certConstr) or 0 == int(certConstr):
        certCont = certCont.split('-----END CERTIFICATE-----')[1]
        certCont += '-----END CERTIFICATE-----'
    certObj = crypto.load_certificate(crypto.FILETYPE_PEM, certCont)
    if 0 == int(keyOnly):
        ASN1 = crypto.dump_certificate(crypto.FILETYPE_ASN1, certObj)
    else:
        pubKeyObj = certObj.get_pubkey()
        ASN1 = crypto.dump_publickey(crypto.FILETYPE_ASN1, pubKeyObj)
    if 1 == int(hashType):
        output = hashlib.sha256(ASN1).hexdigest()
    elif 2 == int(hashType):
        output = hashlib.sha512(ASN1).hexdigest()
    return output.encode()

def tlsaRecordsFromCertFile(certFilenames, tlsaTypes = [[3,0,1], [3,0,2], [3,1,1], [3,1,2], [2,0,1], [2,0,2], [2,1,1], [2,1,2]]):
    tlsaList = []
    if type(certFilenames) is list:
        for e in certFilenames:
            tlsaList.extend(tlsaRecordsFromCertFile(e, tlsaTypes))
        return tlsaList
    tlsaDictList = [{'usage': tlsaType[0], 'selector': tlsaType[1], 'matchingtype': tlsaType[2], 'tlsa': tlsaFromCertFile(certFilenames, tlsaType[0], tlsaType[1], tlsaType[2])} for tlsaType in tlsaTypes]
    return tlsaDictList

def tlsaFromFile(tlsaDict):
    if type(tlsaDict) is list:
        rv = [tlsaFromFile(e) for e in tlsaDict]
        tlsaDict[:] = rv
        return rv
    if 'filename' not in tlsaDict:
        return tlsaDict
    tlsaDict['tlsa'] = tlsaFromCertFile(tlsaDict['filename'], tlsaDict['usage'], tlsaDict['selector'], tlsaDict['matchingtype'])
    tlsaDict['tlsa'] = tlsaDict['tlsa'].replace(b'\n', b'')
    return tlsaDict



#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

def dkimFromFile(dkimDict):
    if type(dkimDict) is list:
        rv = [dkimFromFile(e) for e in dkimDict]
        dkimDict[:] = rv
        return rv
    if 'filename' not in dkimDict:
        return dkimDict
    dkimFilename = dkimDict['filename']
    with open(dkimFilename, 'r') as f:
        dkimCont = f.read()
    for r in ['"', '\n', '\r', '\t', '(', ')', ' ']:
        dkimCont = dkimCont.replace(r, '')
    n = dkimCont.split('._domainkey')[0]
    try:
        v = dkimCont.split('v=')[1]
        v = v.split(';')[0]
    except:
        v = None
    try:
        k = dkimCont.split('k=')[1]
        k = k.split(';')[0]
    except:
        k = None
    try:
        p = dkimCont.split('p=')[1]
        p = p.split(';')[0]
    except:
        p = None
    formFileDict = {'p': p, 'keyname': n, 'v': v, 'k': k}
    for k, v in formFileDict.items():
        if k in dkimDict.keys():
            continue
        dkimDict[k] = v
    return dkimDict






#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import unittest

from dnsuptools import dkimrecgen

class TestDKIMrecgen(unittest.TestCase):
    def testDKIMFromFile1024(self):
        dkim = dkimrecgen.dkimFromFile({'filename': 'tests/testdkimkey1024.txt'})
        self.assertEqual(dkim, {'filename': 'tests/testdkimkey1024.txt', 'p': 'MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDQP7L2UVYrwo6pNHEXzSGrT7B+P8HBTEX6AecR9xTHr+Y9WwiYJAaKlLESqTB5SzQYlO6c+QeOv1fhnExBRxMqQEM0FhpvdwBadYx5Df4bSxt/5fjherBAiE9FbnTWWNDO6isGdAJ75+yWceRvXx0MajqSUwggqaZWNirWJUq34QIDAQAB', 'keyname': 'testdkimkey', 'v': 'DKIM1', 'k': 'rsa'})

    def testDKIMFromFile2048(self):
        dkim = dkimrecgen.dkimFromFile({'filename': 'tests/testdkimkey2048.txt'})
        self.assertEqual(dkim, {'filename': 'tests/testdkimkey2048.txt', 'p': 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAucgtUOS7wajLj5aVk+J/pNjAkx/9qt4s4tuhXRsHcYcroVRHegtDTNUIZdP3oF/QERuBQAE2gfNoV2ZqC9Nop3EXf0fjOFJoUxTyOK2IXMglbOD5+sptNaJwY7OsS+jhjNJBG5Jej7HcR/eqya7kUovVCGqjy6Ii2Tik5Sun0yHz3yqKrQgdq/I3ev7xAfeiBdmL8ZweaVG8aqK51Fbf0j1VPo65F/CcXVVs00urQdkR8hjCKk3l7NMwHI6fmiOViAxMNvpv48A93K9jL3LylPErybww4XBsEMwAODuymrFvXS8e7eM8mPejZP+Me+ngPrASoUal3NCziWKmfpmqKwIDAQAB', 'keyname': 'testdkimkey', 'v': 'DKIM1', 'k': 'rsa'})

    def testDKIMFromFile4096(self):
        dkim = dkimrecgen.dkimFromFile({'filename': 'tests/testdkimkey4096.txt'})
        self.assertEqual(dkim, {'filename': 'tests/testdkimkey4096.txt', 'p': 'MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAxTvW1ZqXNIFEJzpU5xwLPTna84vKpeWM83MS8jf7THtYfNMJmtG706j2fHmWG55uuU0r5fnYaHOBj1IT4qtFlzHDjqzrQYNUpTzsmHqpe3EMec/Zhja5Rzd/HLCZXFzBpe7welj0fYM81StCs1GrDKQvtMkqdP2VfcWNEdXksffXPMK5T5nw4tO1q99xiIGxr4+lBo12zNaUrtv18E7klBGhYkEfA1EPL5oYGKnHaBD935YU7CM0UQP+MbunM0+Tj02EfT2zwpxRkQf9a0nRqvdzgWqEDTleyXiWVp8TKqTKhoSiLmqZPv/c3d+YDAfe2L5cIDgdGRJ0eqrAsKrebf+YmVmQgr3BRfkrXGwfbg2hyqU2VCPf40KG8NOMI/4CAHbkf1RhUzv+1u8uF32xUKrYlUt8TH2BODE4CHg7OzCBCxu0mUJSMEcsgekOALm0ArDWts/8k1Dtk2NKhoHiKCLl3Q/C8W0A4bkjNKj4IEe11UYUXEJdpq0M5JawBQ1rp7YOrRnCMN5YCvIXUwt5S+OEFgmo5hO7NtFGFBfEgdaM21c5dLlpVzaWXDyTW8FFAPCN/xFqR6xhKLTzJppnD2RIvak7hNnEfc6BkNTR1sBzx23jgwfTX2Te287zQRQeORONoARWGFn1UnCz7NeomuOTugEuDudv/sydfX380gMCAwEAAQ==', 'keyname': 'testdkimkey', 'v': 'DKIM1', 'k': 'rsa'})





#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import unittest

from dnsuptools import tlsarecgen

class TestTLSArecgen(unittest.TestCase):
    def testTLSAFromFile301(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 3, 'selector': 0, 'matchingtype': 1})
        self.assertEqual(tlsa['tlsa'], b'edad412aed22e45ef40b0dc8bfae40306b0babba4e0aab0985e5208c6c512fe7')

    def testTLSAFromFile311(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 3, 'selector': 1, 'matchingtype': 1})
        self.assertEqual(tlsa['tlsa'], b'db34027aae2af459f0279bebc049ef2127233b1b9fc23c2223f08cf36f0ee6a5')

    def testTLSAFromFile302(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 3, 'selector': 0, 'matchingtype': 2})
        self.assertEqual(tlsa['tlsa'], b'3e4a29703c33a2ec5cc1c6b798da3bc69510bd16f7a4f3e6747dfa8aa3e029217ee6effec41aea9090bd7e8ea19156492bfe58f1adac71fcfa23d44e98aa8ad2')

    def testTLSAFromFile312(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 3, 'selector': 1, 'matchingtype': 2})
        self.assertEqual(tlsa['tlsa'], b'b157743af61c14cb16f840208cc9d88a1454a76d0e9051f12a530201ca89d10c2c2c876914770299ac5e39706923153ded0c7ea7cb8c4517a0d613230fd79b66')

    def testTLSAFromFile201(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 2, 'selector': 0, 'matchingtype': 1})
        self.assertEqual(tlsa['tlsa'], b'0f39373b3dab935434b5eceab8fd22249a2dcdf90a33fc5a71221d20f2d8e1b9')

    def testTLSAFromFile211(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 2, 'selector': 1, 'matchingtype': 1})
        self.assertEqual(tlsa['tlsa'], b'ceb408dbfd6af22d9244f98c675b0c9ed2241be94c5b495e80f168903a379eb6')

    def testTLSAFromFile202(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 2, 'selector': 0, 'matchingtype': 2})
        self.assertEqual(tlsa['tlsa'], b'6363ee8e61e476c46978e1eb388cfd7ad3a57713d89cad292176bfdb10c3628691c763ed5cbce8f904657a6427bad22ed9106ea0e25428575a5d1ef0fa20b9f5')

    def testTLSAFromFile212(self):
        tlsa = tlsarecgen.tlsaFromFile({'filename': 'tests/fullchain.pem', 'usage': 2, 'selector': 1, 'matchingtype': 2})
        self.assertEqual(tlsa['tlsa'], b'187160f11f4df298647ea2d48504ecf06f32ea255d54642174bff488670108555599424567ba1949df6713ab14947eca724fe73fa9cd14ba166863686fb17728')




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import unittest
from dnsuptools import dnsupdate
from tests.passwords import inwxUserDict, inwxPasswdDict

turl = "test23.bahn.cf"
#turl = "bahn.cf"

def filterResult(x, pkeys, skeys):
    if type(pkeys) is not list:
        pkeys = [pkeys]
    if type(skeys) is not list:
        skeys = [skeys]
    return {'_'.join([str(e[pkey]) for pkey in pkeys]): \
            {k: v for k, v in e.items() if k in skeys} for e in x}

class TestDNSUpdateMiscFncs(unittest.TestCase):
    def testFlatten(self):
        x = [1,2,3,[4,5,[6],7],8]
        y = dnsupdate.flatten(x)
        self.assertEqual(y, [1,2,3,4,5,6,7,8])

    def testExtractIDs(self):
        x = [{'id':1,'xy':'a'},{'id':2},[{'id':3},{'id':4},[{'id':5}],{'id':6}]]
        y = dnsupdate.extractIds(x)
        self.assertEqual(y, [1,2,[3,4,[5],6]])

    def testDefaultDictList(self):
        baseDict = {'x':1,'y':2}
        dictList = [{},{'z':3},{'x':3},{'x':3,'y':4},{'x':5,'y':6,'z':7}]
        y = dnsupdate.defaultDictList(baseDict, dictList)
        self.assertEqual(y, [{'x':1,'y':2},{'x':1,'y':2,'z':3},{'x':3,'y':2},
                             {'x':3,'y':4},{'x':5,'y':6,'z':7}])


    def testMatchUpperLabels(self):
        mul = dnsupdate.MatchUpperLabels()
        fltr = {'name': 'sub.domain.local', 'domain': 'domain.local'}
        x = [{'name': 'sub.domain.local'}, {'name': 'domain.local'},
             {'name': 'very.sub.domain.local'}]
        mul.pre(fltr)
        mul.post(x)
        with self.subTest("MatchUpperLabels.pre(filterDict)"):
            self.assertEqual(fltr, {'domain': 'domain.local'})
        with self.subTest("MatchUpperLabels.stateDict"):
            self.assertEqual(mul.stateDict, {'name': 'sub.domain.local'})
        with self.subTest("MatchUpperLabels.post(rv)"):
            self.assertEqual(x, [{'name': 'sub.domain.local'},
                                 {'name': 'very.sub.domain.local'}])




class TestDNSUpdate(unittest.TestCase):
    def setUp(self):
        self.dnsu = dnsupdate.DNSUpdate()
        self.dnsu.setHandler('inwx')
        self.dnsu.handler.setUserDict(inwxUserDict)
        self.dnsu.handler.setPasswdDict(inwxPasswdDict)
        self.dnsu.delete({'name': turl}, wild=True)

    def testDNSops(self):
        self.dnsu.delete({'name': turl}, wild=True)
        qry = self.dnsu.qry({'name': turl})
        with self.subTest("Query result length after first delete 0"):
            self.assertEqual(len(qry), 0)
        self.dnsu.add({'name': turl, 'type': 'A', 'content': '1.2.3.4'})
        qry = self.dnsu.qry({'name': turl})
        with self.subTest("Query result length after add 1"):
            self.assertEqual(len(qry), 1)
        self.dnsu.add([{'name': turl, 'type': 'NS', 'content': 'ns23.'+turl},
                       {'name': turl, 'type': 'MX', 'content': 'mx23.'+turl}])
        qry = self.dnsu.qry({'name': turl})
        with self.subTest("Query result length after 2 additional adds 3"):
            self.assertEqual(len(qry), 3)
        self.dnsu.delete([{'name': turl, 'type': 'NS'}])
        qry = self.dnsu.qry({'name': turl})
        with self.subTest("Query result length after 1 delete 2"):
            self.assertEqual(len(qry), 2)
        self.dnsu.delete({'name': turl})
        qry = self.dnsu.qry({'name': turl})
        with self.subTest("Query result length after last delete 0"):
            self.assertEqual(len(qry), 0)

    def testUpdateOps(self):
        self.dnsu.delete({'name': turl}, wild=True)
        y = [{'prio': 20, 'content': 'mx20.xmpl'},
             {'prio': 30, 'content': 'mx30.xmpl'},
             {'prio': 40, 'content': 'mx40.xmpl'}]
        self.dnsu.addDictList({'name': turl, 'type': 'MX'}, y)
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        print(q)
        refq = filterResult(q, ['content'], ['name', 'type', 'id', 'prio'])
        ref = {k: v for k, v in refq.items() if k != 'mx20.xmpl'}
        ref['mxup.xmpl'] = refq['mx20.xmpl']
        self.dnsu.update({'name': turl, 'type': 'MX', 'prio': 20},
                         {'content': 'mxup.xmpl'})
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        print(q)
        res = filterResult(q, ['content'], ['name', 'type', 'id', 'prio'])
        with self.subTest("Check update() - id and content"):
            self.assertEqual(res, ref)
        ref = {k: v for k, v in refq.items() if not (k == 'mx30.xmpl' or k ==
                                                     'mx20.xmpl')}
        ref['mxup30.xmpl'] = refq['mx30.xmpl']
        ref['mxup.xmpl'] = refq['mx20.xmpl']
        self.dnsu.update({'name': turl, 'type': 'MX', 'content': 'mx30.xmpl'},
                         {'content': 'mxup30.xmpl'})
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        print(q)
        res = filterResult(q, ['content'], ['name', 'type', 'id', 'prio'])
        with self.subTest("Check update() - preserve prio in updated record"):
            self.assertEqual(res, ref)


    def testListOps(self):
        self.dnsu.delList({'name': turl, 'type': 'MX'})
        x = [{'name': turl, 'type': 'MX', 'prio': 10, 'content': 'mx23.xmpl'},
             {'name': turl, 'type': 'MX', 'prio': 10, 'content': 'mx42.xmpl'}]
        y = [{'prio': 20, 'content': 'mx20.xmpl'},
             {'prio': 30, 'content': 'mx30.xmpl'},
             {'prio': 40, 'content': 'mx40.xmpl'}]
        contx = [e['content'] for e in x]
        self.dnsu.addList({'name': turl, 'type': 'MX', 'prio': 10}, contx)
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        res = filterResult(q, ['name', 'content'], ['type', 'content', 'prio'])
        ref = filterResult(x, ['name', 'content'], ['type', 'content', 'prio'])
        with self.subTest("Check addList()"):
            self.assertEqual(res, ref)
        self.dnsu.addDictList({'name': turl, 'type': 'MX'}, y)
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        res = filterResult(q, ['content'], ['content', 'prio'])
        ref = filterResult(x+y, ['content'], ['content', 'prio'])
        with self.subTest("Check addDictList()"):
            self.assertEqual(res, ref)
        self.dnsu.delList({'name': turl}, ['mx30.xmpl', 'mx40.xmpl'])
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        res = filterResult(q, ['content'], ['content', 'prio'])
        ref = filterResult(x+[y[0]], ['content'], ['content', 'prio'])
        with self.subTest("Check delList() (wo preserve)"):
            self.assertEqual(res, ref)
        self.dnsu.delList({'name': turl}, '*', ['mx20.xmpl', 'mx42.xmpl'])
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        idPresPre = [e['id'] for e in q if e['content'] == 'mx42.xmpl']
        res = filterResult(q, ['content'], ['content', 'prio'])
        ref = filterResult([x[1],y[0]], ['content'], ['content', 'prio'])
        with self.subTest("Check delList() (with preserve)"):
            self.assertEqual(res, ref)
        self.dnsu.setList({'name': turl, 'type': 'MX', 'prio': 10}, contx)
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        idPresPost = [e['id'] for e in q if e['content'] == 'mx42.xmpl']
        res = filterResult(q, ['content'], ['content', 'prio'])
        ref = filterResult(x+[y[0]], ['content'], ['content', 'prio'])
        with self.subTest("Check setList() (preserve)"):
            self.assertEqual(res, ref)
        with self.subTest("Check setList() (id preserved)"):
            self.assertEqual(idPresPost, idPresPre)
        self.dnsu.setList({'name': turl, 'type': 'MX', 'prio': 10}, [contx[1]])
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        idPresPre3 = [e['id'] for e in q if e['content'] == 'mx20.xmpl']
        idPresPost2 = [e['id'] for e in q if e['content'] == 'mx42.xmpl']
        res = filterResult(q, ['content'], ['content', 'prio'])
        ref = filterResult([x[1],y[0]], ['content'], ['content', 'prio'])
        with self.subTest("Check setList() (delete)"):
            self.assertEqual(res, ref)
        with self.subTest("Check setList() (id preserved after delete)"):
            self.assertEqual(idPresPost2, idPresPost)
        self.dnsu.setDictList({'name': turl, 'type': 'MX'}, [{}], y)
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        idPresPost3 = [e['id'] for e in q if e['content'] == 'mx20.xmpl']
        res = filterResult(q, ['content'], ['content', 'prio'])
        ref = filterResult(y, ['content'], ['content', 'prio'])
        with self.subTest("Check setDictList() (delete and preserve)"):
            self.assertEqual(res, ref)
        with self.subTest("Check setDictList() (id preserved after delete)"):
            self.assertEqual(idPresPre3, idPresPost3)
        self.dnsu.setDictList({'name': turl}, [{'prio': 30}, {'prio': 40}], x)
        q = self.dnsu.qry({'name': turl, 'type': 'MX'})
        idPresPost4 = [e['id'] for e in q if e['content'] == 'mx20.xmpl']
        res = filterResult(q, ['content'], ['content', 'prio'])
        ref = filterResult(x+[y[0]], ['content'], ['content', 'prio'])
        with self.subTest("Check setDictList() (selective delete and set)"):
            self.assertEqual(res, ref)
        with self.subTest("Check setDictList() (id preserved after set)"):
            self.assertEqual(idPresPost3, idPresPost4)



    def testWildcards(self):
        self.dnsu.delete({'name': turl}, [], True)
        recsAdded = [{'name':'text.ns42.'+turl,'type':'TXT','content':'x'},
                     {'name':'ns42.'+turl,'type':'NS','content':'ns23.xmpl'},
                     {'name':'mx42.'+turl,'type':'MX','content':'mx23.xmpl'},
                     {'name':turl,'type':'A','content':'1.2.3.4'}]
        recsDeled = [{'name': turl,'type': 'NS'},
                     {'name': turl, 'content': 'mx23.xmpl'},
                     {'name': turl, 'content': '1.2.3.4', 'type': 'TXT'}]
        recsRem = recsAdded[1:3]
        recUpd = {'name': 'mx42.'+turl, 'type': 'MX', 'content': 'mx1337.xmpl'}
        recsNew = [recsAdded[1], recUpd]
        self.dnsu.add(recsAdded)
        qry = self.dnsu.qryWild({'name': turl})
        names = {e['name'] for e in qry}
        with self.subTest("Check if all names are there"):
            self.assertEqual(names, {'text.ns42.'+turl,'ns42.'+turl,
                                     'mx42.'+turl,turl})
        recsAddedDict = filterResult(recsAdded, 'name', ['type', 'content'])
        recsQrydDict = filterResult(qry, 'name', ['type', 'content'])
        with self.subTest("Check if name, type and content are correct"):
            self.assertEqual(recsQrydDict, recsAddedDict)
        self.dnsu.delete({'name': turl}, recsDeled, True)
        qry = self.dnsu.qryWild({'name': turl})
        recsQrydDict = filterResult(qry, 'name', ['type', 'content'])
        recsRemDict = filterResult(recsRem, 'name', ['type', 'content'])
        with self.subTest("Are name, type, content correct, after preserve?"):
            self.assertEqual(recsQrydDict, recsRemDict)
        self.dnsu.update({'name': 'mx42.'+turl}, recUpd)
        qry = self.dnsu.qryWild({'name': turl})
        recsQrydDict = filterResult(qry, 'name', ['type', 'content'])
        recsNewDict = filterResult(recsNew, 'name', ['type', 'content'])
        with self.subTest("Are name, typ, content correct, after update?"):
            self.assertEqual(recsQrydDict, recsNewDict)




    def tearDown(self):
        self.dnsu.handler.disconnect()




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import setuptools
import dnsuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dnsuptools",
    version=dnsuptools.__version__,
    author="Stefan Helmert",
    author_email="stefan.helmert@t-online.de",
    description="Library interfaces to dns service api for query, update, add, delete comfortably.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TheTesla/dnsuptools",
    packages=setuptools.find_packages(),
    install_requires=[
        "simpleloggerplus",
        "dnspython",
        "pycurl",
        "inwx-domrobot"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)

